@@ -11,7 +11,7 @@ import fnmatch,re,string
 from dnslib.bit import get_bits,set_bits
 from dnslib.buffer import Buffer, BufferError
 
-# In theory valid label characters should be letters,digits,hyphen,underscore (LDH) 
+# In theory valid label characters should be letters,digits,hyphen,underscore (LDH)
 # LDH = set(bytearray(b'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_'))
 # For compatibility we only escape non-printable characters
 LDH = set(range(33,127))
@@ -57,7 +57,7 @@ class DNSLabel(object):
     >>> l3.matchGlob("*.[abc]xx.bbb.ccc")
     False
 
-    # Too hard to get unicode doctests to work on Python 3.2  
+    # Too hard to get unicode doctests to work on Python 3.2
     # (works on 3.3)
     # >>> u1 = DNSLabel(u'\u2295.com')
     # >>> u1.__str__() == u'\u2295.com.'
@@ -68,7 +68,7 @@ class DNSLabel(object):
     """
     def __init__(self,label):
         """
-            Create DNS label instance 
+            Create DNS label instance
 
             Label can be specified as:
             - a list/tuple of byte strings
@@ -96,7 +96,7 @@ class DNSLabel(object):
 
     def add(self,name):
         """
-            Prepend name to label 
+            Prepend name to label
         """
         new = DNSLabel(name)
         if self.label:
@@ -110,7 +110,7 @@ class DNSLabel(object):
 
     def matchSuffix(self,suffix):
         """
-            Return True if label suffix matches 
+            Return True if label suffix matches
         """
         suffix = DNSLabel(suffix)
         return self.label[-len(suffix.label):] == suffix.label
@@ -143,7 +143,7 @@ class DNSLabel(object):
         return "<DNSLabel: '%s'>" % str(self)
 
     def __hash__(self):
-        return hash(self.label)
+        return hash(tuple(map(lambda x:x.lower(),self.label)))
 
     def __ne__(self,other):
         return not self == other
@@ -239,13 +239,13 @@ class DNSBuffer(Buffer):
                 pointer = get_bits(self.unpack("!H")[0],0,14)
                 save = self.offset
                 if last == save:
-                    raise BufferError("Recursive pointer in DNSLabel [offset=%d,pointer=%d,length=%d]" % 
+                    raise BufferError("Recursive pointer in DNSLabel [offset=%d,pointer=%d,length=%d]" %
                             (self.offset,pointer,len(self.data)))
                 if pointer < self.offset:
                     self.offset = pointer
                 else:
                     # Pointer can't point forwards
-                    raise BufferError("Invalid pointer in DNSLabel [offset=%d,pointer=%d,length=%d]" % 
+                    raise BufferError("Invalid pointer in DNSLabel [offset=%d,pointer=%d,length=%d]" %
                             (self.offset,pointer,len(self.data)))
                 label.extend(self.decode_name(save).label)
                 self.offset = save
@@ -291,7 +291,7 @@ class DNSBuffer(Buffer):
 
     def encode_name_nocompress(self,name):
         """
-            Encode and store label with no compression 
+            Encode and store label with no compression
             (needed for RRSIG)
         """
         if not isinstance(name,DNSLabel):
@@ -308,5 +308,5 @@ class DNSBuffer(Buffer):
         self.append(b'\x00')
 
 if __name__ == '__main__':
-    import doctest
-    doctest.testmod()
+    import doctest,sys
+    sys.exit(0 if doctest.testmod().failed == 0 else 1)
